@@ -3,7 +3,10 @@ use strict;
 use warnings;
 use 5.010;
 
-our $VERSION = '0.15';
+our $VERSION = '0.24';
+
+our $PANDOC_VERSION;    # a string like '1.16'
+$PANDOC_VERSION ||= $ENV{PANDOC_VERSION};
 
 our %ELEMENTS = (
 
@@ -34,11 +37,12 @@ our %ELEMENTS = (
     Cite        => [ Inline => qw(citations content) ],
     Code        => [ Inline => qw(attr content) ],
     Space       => ['Inline'],
+    SoftBreak   => ['Inline'],
     LineBreak   => ['Inline'],
     Math        => [ Inline => qw(type content) ],
     RawInline   => [ Inline => qw(format content) ],
-    Link        => [ Inline => qw(content target) ],
-    Image       => [ Inline => qw(content target) ],
+    Link        => [ Inline => qw(attr content target) ],
+    Image       => [ Inline => qw(attr content target) ],
     Note        => [ Inline => 'content' ],
     Span        => [ Inline => qw(attr content) ],
 
@@ -64,11 +68,14 @@ foreach (
 
 use Carp;
 use JSON qw(decode_json);
-use Scalar::Util qw(reftype);
+use Scalar::Util qw(blessed reftype);
 use Pandoc::Walker qw(walk);
 
 use parent 'Exporter';
-our @EXPORT = ( keys %ELEMENTS, qw(Document attributes citation pandoc_json) );
+our @EXPORT = (
+    keys %ELEMENTS,
+    qw(Document attributes metadata citation pandoc_json)
+);
 our @EXPORT_OK = ( @EXPORT, 'element' );
 
 # create constructor functions
@@ -88,18 +95,20 @@ foreach my $name ( keys %ELEMENTS ) {
         sub {
             croak "$name expects $numargs arguments, but given " . scalar @_
               if @_ != $numargs;
-            bless { t => $name, c => ( @_ == 1 ? $_[0] : [@_] ) }, $class;
+            my $self = bless { t => $name, c => ( @_ == 1 ? $_[0] : [@_] ) }, $class;
+            $self->set_content(@_);
+            $self;
         },
         '$' x $numargs
     );
 
     for ( my $i = 0 ; $i < @accessors ; $i++ ) {
-        my $code = @accessors == 1 ? "\$_[0]->{c}" : "\$_[0]->{c}->[$i]";
-
+        my $member = @accessors == 1 ? "\$e->{c}" : "\$e->{c}->[$i]";
+        my $code = "my \$e = shift; $member = ( 1 == \@_ ? \$_[0] : [\@_] ) if \@_; return";
         # auto-bless on access via accessor (TODO: move to constructor?)
-        if ( $accessors[$i] =~ s/:\[(.+)\]$// ) {
-            $code = "[ map { bless \$_, 'Pandoc::Document::$1' } \@{$code} ]";
-        }
+        $code .= $accessors[$i] =~ s/:\[(.+)\]$//
+        ? " [ map { bless \$_, 'Pandoc::Document::$1' } \@{$member} ];"
+        : " $member;";
         for ( split '/', $accessors[$i] ) {
             *{ $class . "::$_" } = eval "sub { $code }";
         }
@@ -116,31 +125,23 @@ sub element {
 
 sub Document($$) {
     @_ == 2 or croak "Document expects 2 arguments, but given " . scalar @_;
-    return bless [ { unMeta => $_[0] }, $_[1] ], 'Pandoc::Document';
+    my $meta = metadata(shift);
+    return bless [ { unMeta => $meta }, shift ], 'Pandoc::Document';
 }
 
 # specific accessors
 
-sub Pandoc::Document::Link::url                   { $_[0]->{c}->[1][0] }
-sub Pandoc::Document::Link::title                 { $_[0]->{c}->[1][1] }
-sub Pandoc::Document::Image::url                  { $_[0]->{c}->[1][0] }
-sub Pandoc::Document::Image::title                { $_[0]->{c}->[1][1] }
 sub Pandoc::Document::DefinitionPair::term        { $_[0]->[0] }
 sub Pandoc::Document::DefinitionPair::definitions { $_[0]->[1] }
 
 # additional functions
 
 sub attributes($) {
-    my ($attrs) = @_;
-    return [
-        defined $attrs->{id}      ? $attrs->{id}      : '',
-        defined $attrs->{classes} ? $attrs->{classes} : [],
-        [
-            map { $_ => $attrs->{$_} }
-              grep { $_ ne 'id' and $_ ne 'classes' }
-              keys %$attrs
-        ]
-    ];
+
+    my $e = Span(['',[],[]],[]); # to make use of AttributesRole
+    $e->keyvals(@_);
+
+    return $e->attr;
 }
 
 sub citation($) {
@@ -158,7 +159,21 @@ sub citation($) {
     };
 }
 
+sub metadata($) {
+    $_[0]; # TODO: issue #10 and #34
+}
+
 sub pandoc_json($) {
+    state $ast_to_element = sub {   # compile once, use repeatedly, keep private
+        my $class = 'Pandoc::Document::' . $_[0]->{t};
+        if ( 'MetaMap' eq $_[0]->{t} ) {
+            for my $v ( values %{ $_[0]->{c} } ) {
+                $_[1]->( $v, $_[1] );
+            }
+        }
+        return $class->new_from_ast( $_[0] );
+    };
+
     shift if $_[0] =~ /^Pandoc::/;
 
     my $ast = eval { decode_json( $_[0] ) };
@@ -170,15 +185,19 @@ sub pandoc_json($) {
     return unless reftype $ast;
 
     if ( reftype $ast eq 'ARRAY' ) {
-        $ast = Document( $ast->[0]->{unMeta}, $ast->[1] );
+        my $meta = $ast->[0]->{unMeta};
+        for my $v ( values %$meta ) {
+            $v = $ast_to_element->( $v, $ast_to_element );
+        }
+        $ast = Document( $meta, $ast->[1] );
     }
-    elsif ( reftype $ast eq 'HASH' ) {
-        $ast = element( $ast->{t}, $ast->{c} );
+    elsif ( reftype $ast eq 'HASH' and $ast->{t} ) {
+
+        # $ast = element( $ast->{t}, $ast->{c} );
+        $ast = $ast_to_element->( $ast, $ast_to_element );
     }
 
-    walk $ast, sub {
-        bless $_[0], 'Pandoc::Document::' . $_[0]->{t};
-    };
+    walk $ast, $ast_to_element;
 
     return $ast;
 }
@@ -190,12 +209,24 @@ sub pandoc_json($) {
     package Pandoc::Document;
     use strict;
     our $VERSION = '0.04';
-    our @ISA     = ('Pandoc::Document::Element');
-    sub TO_JSON     { [ @{ $_[0] } ] }
-    sub name        { 'Document' }
-    sub meta        { $_[0]->[0]->{unMeta} }
-    sub content     { $_[0]->[1] }
+    our @ISA = ('Pandoc::Document::Element');
+    sub name { 'Document' }
+    sub meta {
+        $_[0]->[0]->{unMeta} = Pandoc::Elements::metadata($_[1]) if @_ > 1;
+        $_[0]->[0]->{unMeta}
+    }
+    sub content { 
+        $_[0]->[1] = $_[1] if @_ > 1; 
+        $_[0]->[1] 
+    }
     sub is_document { 1 }
+    sub metavalue {
+        my $meta = $_[0]->meta;
+        return { map { $_ => $meta->{$_}->metavalue } keys %$meta }
+    }
+    sub string {
+        join '', map { $_->string } @{$_[0]->content}
+    }
 }
 
 {
@@ -205,15 +236,55 @@ sub pandoc_json($) {
     use warnings;
     our $VERSION = $Pandoc::Document::VERSION;
     use JSON ();
-    use Scalar::Util qw(reftype);
+    use Scalar::Util qw(reftype blessed);
     use Pandoc::Walker ();
+    use subs qw(walk query transform);    # Silence syntax warnings
 
     sub to_json {
-        JSON->new->utf8->convert_blessed->encode( $_[0] );
+        JSON->new->utf8->canonical->convert_blessed->encode( $_[0] );
+    }
+
+    sub TO_JSON {
+
+        # Run everything thru this method so arrays/hashes are cloned
+        # and objects without TO_JSON methods are stringified.
+        # Required to ensure correct scalar types for Pandoc.
+
+# There is no easy way in Perl to tell if a scalar value is already a string or number,
+# so we stringify all scalar values and numify/boolify as needed afterwards.
+
+        my ( $ast, $maybe_blessed ) = @_;
+        if ( $maybe_blessed && blessed $ast ) {
+            return $ast if $ast->can('TO_JSON');    # JSON.pm will convert
+                 # may have overloaded stringification! Should we check?
+                 # require overload;
+              # return "$ast" if overload::Method($ast, q/""/) or overload::Method($ast, q/0+/);
+              # carp "Non-stringifiable object $ast";
+            return "$ast";
+        }
+        elsif ( 'ARRAY' eq reftype $ast ) {
+            return [ map { ref($_) ? TO_JSON( $_, 1 ) : "$_"; } @$ast ];
+        }
+        elsif ( 'HASH' eq reftype $ast ) {
+            my %ret = %$ast;
+            while ( my ( $k, $v ) = each %ret ) {
+                $ret{$k} = ref($v) ? TO_JSON( $v, 1 ) : "$v";
+            }
+            return \%ret;
+        }
+        else { return "$ast" }
+    }
+
+    sub name        { $_[0]->{t} }
+    sub content     {
+       my $e = shift;
+       $e->set_content(@_) if @_;
+       $e->{c}
+    }
+    sub set_content { # TODO: document this
+       my $e = shift;
+       $e->{c} = @_ == 1 ? $_[0] : [@_]
     }
-    sub TO_JSON { return { %{ $_[0] } } }
-    sub name    { $_[0]->{t} }
-    sub content { $_[0]->{c} }
     sub is_document { 0 }
     sub is_block    { 0 }
     sub is_inline   { 0 }
@@ -222,20 +293,22 @@ sub pandoc_json($) {
     *query     = *Pandoc::Walker::query;
     *transform = *Pandoc::Walker::transform;
 
+    sub new_from_ast { bless $_[1] => $_[0] }
+
     sub string {
 
         # TODO: fix issue #4 to avoid this duplication
-        if ( $_[0]->name =~ /^(Str|Code|Math)$/ ) {
+        if ( $_[0]->name =~ /^(Str|Code|Math|MetaString)$/ ) {
             return $_[0]->content;
         }
-        elsif ( $_[0]->name =~ /^(LineBreak|Space)$/ ) {
+        elsif ( $_[0]->name =~ /^(LineBreak|SoftBreak|Space)$/ ) {
             return ' ';
         }
         join '', @{
             $_[0]->query(
                 {
-                    'Str|Code|Math'   => sub { $_->content },
-                    'LineBreak|Space' => sub { ' ' },
+                    'Str|Code|Math|MetaString'  => sub { $_->content },
+                    'LineBreak|Space|SoftBreak' => sub { ' ' },
                 }
             );
         };
@@ -270,16 +343,72 @@ sub pandoc_json($) {
         return 0 unless $self->can('match_attributes');
         return $self->match_attributes($selector);
     }
+
 }
 
 {
 
     package Pandoc::Document::AttributesRole;
-    my $IDENTIFIER = qr{\p{L}(\p{L}|[0-9_:.-])*};
-    sub id      { $_[0]->attr->[0] }
-    sub classes { $_[0]->attr->[1] }
-    sub class   { join ' ', @{ $_[0]->classes } }
+    use Hash::MultiValue;
+    use Scalar::Util qw(reftype blessed);
+    use Carp qw(croak);
 
+    my $IDENTIFIER = qr{\p{L}(\p{L}|[0-9_:.-])*};
+
+    sub id {
+        $_[0]->attr->[0] = "$_[1]" if @_ > 1;
+        $_[0]->attr->[0]
+    }
+
+    sub classes {
+        my $e = shift;
+        croak 'Method classes() is not a setter' if @_;
+        $e->attr->[1]
+    }
+
+    sub class {
+        my $e = shift;
+        if (@_) {
+            $e->attr->[1] = [
+                grep { $_ ne '' }
+                map { split qr/\s+/, $_ }
+                map { (ref $_ and reftype $_ eq 'ARRAY') ? @$_ : $_ }
+                @_
+            ];
+        }
+        join ' ', @{$e->classes}
+    }
+
+    sub add_attribute {
+        my ($e, $key, $value) = @_;
+        if ($key eq 'id') {
+            $e->id($value);
+        } elsif ($key eq 'class') {
+            $value = ["$value"] unless (reftype $value // '') eq 'ARRAY';
+            push @{$e->attr->[1]}, grep { $_ ne '' } map { split qr/\s+/, $_ } @$value;
+        } else {
+            push @{$e->attr->[2]}, [ $key, "$value" ];
+        }
+    }
+
+    sub keyvals {
+        my $e = shift;
+        if (@_) {
+            my $attrs = @_ == 1 ? shift : Hash::MultiValue->new(@_);
+            unless (blessed $attrs and $attrs->isa('Hash::MultiValue')) {
+                $attrs = Hash::MultiValue->new(%$attrs);
+            }
+            $e->attr->[1] = [] if exists $attrs->{class};
+            $e->attr->[2] = [];
+            $attrs->each(sub { $e->add_attribute(@_) });
+        }
+        my @h;
+        push @h, id => $e->id if $e->id ne '';
+        push @h, class => $e->class if @{$e->classes};
+        Hash::MultiValue->new( @h, map { @$_ } @{$e->attr->[2]} );
+    }
+
+    # TODO: rename and/or extend to keyvals check
     sub match_attributes {
         my ( $self, $selector ) = @_;
         $selector =~ s/^\s+|\s+$//g;
@@ -319,11 +448,133 @@ sub pandoc_json($) {
 {
 
     package Pandoc::Document::Meta;
+    use Scalar::Util 'reftype';
     our $VERSION = $PANDOC::Document::VERSION;
     our @ISA     = ('Pandoc::Document::Element');
     sub is_meta { 1 }
 }
 
+{
+
+    package Pandoc::Document::LinkageRole;
+    our $VERSION = $PANDOC::Document::VERSION;
+
+    for my $Element (qw[ Link Image ]) {
+        no strict 'refs';    #no critic
+        unshift @{"Pandoc::Document::${Element}::ISA"}, __PACKAGE__; # no critic
+    }
+
+    sub url   { $_[0]->{c}->[-1][0] }
+    sub title { $_[0]->{c}->[-1][1] }
+
+    sub new_from_ast {
+        my ( $class, $ast ) = @_;
+        if ( 2 == @{ $ast->{c} } ) {
+
+            # prepend attributes to old-style ast
+            unshift @{ $ast->{c} }, [ "", [], [] ];
+        }
+        return bless $ast => $class;
+    }
+}
+
+# Special TO_JSON methods to coerce data to int/number/Boolean as appropriate
+# and to downgrade document model for Pandoc < 1.16
+
+sub Pandoc::Document::SoftBreak::TO_JSON {
+    if ( $Pandoc::Elements::PANDOC_VERSION
+        and ( $Pandoc::Elements::PANDOC_VERSION lt '1.16' ) ) {
+        return { t => 'Space', c => [] };
+    } else {
+        return { t => 'SoftBreak', c => [] };
+    }
+}
+
+sub Pandoc::Document::LinkageRole::TO_JSON {
+    my $ast = Pandoc::Document::Element::TO_JSON( $_[0] );
+    if ( $Pandoc::Elements::PANDOC_VERSION
+        and ( $Pandoc::Elements::PANDOC_VERSION lt '1.16' ) )
+    {
+        # remove attributes from new-style ast
+        $ast->{c} = [ @{ $ast->{c} }[ 1, 2 ] ];
+    }
+    return $ast;
+}
+
+sub Pandoc::Document::Header::TO_JSON {
+    my $ast = Pandoc::Document::Element::TO_JSON( $_[0] );
+
+    # coerce heading level to int
+    $ast->{c}[0] = int( $ast->{c}[0] );
+    return $ast;
+}
+
+sub Pandoc::Document::OrderedList::TO_JSON {
+    my $ast = Pandoc::Document::Element::TO_JSON( $_[0] );
+
+    # coerce first item number to int
+    $ast->{c}[0][0] = int( $ast->{c}[0][0] );
+    return $ast;
+}
+
+sub Pandoc::Document::Table::TO_JSON {
+    my $ast = Pandoc::Document::Element::TO_JSON( $_[0] );
+
+    # coerce column widths to numbers (floats)
+    $_ += 0 for @{ $ast->{c}[2] };    # faster than map
+    return $ast;
+}
+
+sub Pandoc::Document::MetaBool::set_content {
+    $_[0]->{c} = $_[1] && $_[1] ne 'false' && $_[1] ne 'FALSE'
+}
+
+sub Pandoc::Document::MetaBool::TO_JSON {
+    return {
+        t => 'MetaBool',
+
+        # coerce Bool value to JSON Boolean object
+        c => $_[0]->{c} ? JSON::true() : JSON::false(),
+    };
+}
+
+sub Pandoc::Document::MetaBool::metavalue {
+    $_[0]->{c} ? 1 : 0
+}
+
+sub Pandoc::Document::MetaMap::metavalue {
+    my $map = $_[0]->{c};
+    return { map { $_ => $map->{$_}->metavalue } keys %$map }
+}
+
+sub Pandoc::Document::MetaInlines::metavalue {
+    join '', map { $_->string } @{$_[0]->{c}}
+}
+
+sub Pandoc::Document::MetaBlocks::metavalue {
+    join "\n", map { $_->string } @{$_[0]}->{c}
+}
+
+sub Pandoc::Document::MetaList::metavalue {
+    [ map { $_->metavalue } @{$_[0]->{c}} ]
+}
+
+sub Pandoc::Document::MetaString::metavalue {
+    $_[0]->{c}
+}
+
+sub Pandoc::Document::Cite::TO_JSON {
+    my $ast = Pandoc::Document::Element::TO_JSON( $_[0] );
+    for my $citation ( @{ $ast->{c}[0] } ) {
+        for my $key (qw[ citationHash citationNoteNum ]) {
+
+            # coerce to int
+            $citation->{$key} = int( $citation->{$key} );
+        }
+    }
+    return $ast;
+}
+
 1;
 __END__
 
@@ -337,8 +588,8 @@ Pandoc::Elements - create and process Pandoc documents
 
 # STATUS
 
-[![Build Status](https://travis-ci.org/nichtich/Pandoc-Elements.png)](https://travis-ci.org/nichtich/Pandoc-Elements)
-[![Coverage Status](https://coveralls.io/repos/nichtich/Pandoc-Elements/badge.png)](https://coveralls.io/r/nichtich/Pandoc-Elements)
+[![Build Status](https://travis-ci.org/nichtich/Pandoc-Elements.svg)](https://travis-ci.org/nichtich/Pandoc-Elements)
+[![Coverage Status](https://coveralls.io/repos/nichtich/Pandoc-Elements/badge.svg)](https://coveralls.io/r/nichtich/Pandoc-Elements)
 [![Kwalitee Score](http://cpants.cpanauthors.org/dist/Pandoc-Elements.png)](http://cpants.cpanauthors.org/dist/Pandoc-Elements)
 
 =end markdown
@@ -350,8 +601,8 @@ The output of this script C<hello.pl>
     use Pandoc::Elements;
     use JSON;
 
-    print Document({ 
-            title => MetaInlines [ Str "Greeting" ] 
+    print Document({
+            title => MetaInlines [ Str "Greeting" ]
         }, [
             Header( 1, attributes { id => 'top' }, [ Str 'Hello' ] ),
             Para [ Str 'Hello, world!' ],
@@ -372,7 +623,7 @@ an equivalent Pandoc Markdown document would be
 Pandoc::Elements provides utility functions to create abstract syntax trees
 (AST) of L<Pandoc|http://pandoc.org/> documents. Pandoc can convert the
 resulting data structure to many other document formats, such as HTML, LaTeX,
-ODT, and ePUB. 
+ODT, and ePUB.
 
 Please make sure to use at least Pandoc 1.12 when processing documents
 
@@ -386,13 +637,13 @@ The following functions and keywords are exported by default:
 
 =over
 
-=item 
+=item
 
 Constructors for all Pandoc document element (L<block elements|/BLOCK ELEMENTS>
 such as C<Para> and L<inline elements|/INLINE ELEMENTS> such as C<Emph>,
 L<metadata elements|/METADATA ELEMENTS> and the L<DOCUMENT ELEMENT/Document>).
 
-=item 
+=item
 
 L<Type keywords|/TYPE KEYWORDS> such as C<Decimal> and C<LowerAlpha> to be used
 as types in other document elements.
@@ -407,20 +658,20 @@ C<element>.
 =head3 pandoc_json $json
 
 Parse a JSON string, as emitted by pandoc in JSON format. This is the reverse
-to method C<to_json>.
+to method C<to_json> but it can read both old (before Pandoc 1.16) and new
+format.
 
 =head3 attributes { key => $value, ... }
 
-Maps a hash reference into an attributes list with id, classes, and ordered
-key-value pairs. The special keys C<id> and C<classes> are recognized but
-setting multi-value attributes or controlled order is not supported with this
-function. You can always manually create an attributes structure:
+Maps a hash reference or instance of L<Hash::MultiValue> into an attributes
+list with id, classes, and ordered key-value pairs. The special keys C<id>
+(string), and C<class> (string or array reference with space-separated class
+names) are recognized. You can always manually create an attributes structure:
 
-    [ $id, [ @classes ], [ key => $value, ... ] ]
+    [ $id, [ @classes ], [ [ key => $value ], ... ] ]
 
-Elements with attributes (element accessor method C<attr>) also provide the
-accessor method C<id>, C<classes>, and C<class>. See L<Hash::MultiValue> for
-easy access to key-value-pairs.
+See also L<attribute methods|/ATTRIBUTE METHODS> to get and set element
+attributes.
 
 =head3 citation { ... }
 
@@ -428,14 +679,14 @@ A citation as part of document element L<Cite|/Cite> must be a hash reference
 with fields C<citationID> (string), C<citationPrefix> (list of L<inline
 elements|/INLINE ELEMENTS>) C<citationSuffix> (list of L<inline
 elements|/INLINE ELEMENTS>), C<citationMode> (one of C<NormalCitation>,
-C>AuthorInText>, C<SuppressAuthor>), C<citationNoteNum> (integer), and
+C<AuthorInText>, C<SuppressAuthor>), C<citationNoteNum> (integer), and
 C<citationHash> (integer). The helper method C<citation> can be used to
 construct such hash by filling in default values and using shorter field names
 (C<id>, C<prefix>, C<suffix>, C<mode>, C<note>, and C<hash>):
 
-    citation { 
-        id => 'foo', 
-        prefix => [ Str "see" ], 
+    citation {
+        id => 'foo',
+        prefix => [ Str "see" ],
         suffix => [ Str "p.", Space, Str "42" ]
     }
 
@@ -448,7 +699,7 @@ construct such hash by filling in default values and using shorter field names
 Create a Pandoc document element of arbitrary name. This function is only
 exported on request.
 
-=head1 ELEMENTS 
+=head1 ELEMENTS
 
 Document elements are encoded as Perl data structures equivalent to the JSON
 structure, emitted with pandoc output format C<json>. All elements are blessed
@@ -462,7 +713,12 @@ methods specific to each element.
 Return the element as JSON encoded string. The following are equivalent:
 
     $element->to_json;
-    JSON->new->utf8->convert_blessed->encode($element);
+    JSON->new->utf8->canonical->convert_blessed->encode($element);
+
+Note that the JSON format changed from Pandoc 1.15 to Pandoc 1.16 by introduction
+of attributes to L<Link|/Link> and L<Image|/Image> elements. Since Pandoc::Elements
+0.16 the new format is serialized by default. Set the package variable or
+environment variable C<PANDOC_VERSION> to 1.15 or below to use the old format.
 
 =head3 name
 
@@ -473,8 +729,8 @@ Return the name of the element, e.g. "Para" for a L<paragraph element|/Para>.
 Return the element content. For most elements (L<Para|/Para>, L<Emph|/Emph>,
 L<Str|/Str>...) the content is an array reference with child elements. Other
 elements consist of multiple parts; for instance the L<Link|/Link> element has
-a link text (C<content>) and a link target (C<target>) with C<url> and
-C<title>.
+attributes (C<attr>, C<id>, C<class>, C<classes>, C<keyvals>) a link text
+(C<content>) and a link target (C<target>) with C<url> and C<title>.
 
 =head3 is_block
 
@@ -508,6 +764,15 @@ Transform the element tree with L<Pandoc::Walker>
 
 Returns a concatenated string of element content, leaving out all formatting.
 
+=head3 ATTRIBUTE METHODS
+
+Elements with attributes (element accessor method C<attr>) also provide the
+getter methods C<id>, C<classes>, C<class>, C<keyvals>, and setter methods
+C<id>, C<class>, C<keyvals>, C<add_attribute>. Method C<keyvals> returns a copy
+of id, class, and attribute key-value pairs as L<Hash::MultiValue>.  If used as
+setter, all key-value pairs (and optionally id and class if given) are
+replaced.
+
 =head2 BLOCK ELEMENTS
 
 =head3 BlockQuote
@@ -525,7 +790,8 @@ L<blocks|/BLOCK ELEMENTS>
 
 =head3 CodeBlock
 
-Code block (literal string C<content>) with attributes (C<attr>)
+Code block (literal string C<content>) with attributes (C<attr>, C<id>,
+C<class>, C<classes>, C<keyvals>)
 
     CodeBlock $attributes, $content
 
@@ -544,14 +810,14 @@ or more definitions (C<definitions>, a list of L<blocks|/BLOCK ELEMENTS>).
 =head3 Div
 
 Generic container of L<blocks|/BLOCK ELEMENTS> (C<content>) with attributes
-(C<attr>).
+(C<attr>, C<id>, C<class>, C<classes>, C<keyvals>).
 
     Div $attributes, [ @blocks ]
 
 =head3 Header
 
-Header with C<level> (integer), attributes (C<attr>), and text (C<content>, a
-list of L<inlines|/INLINE ELEMENTS>).
+Header with C<level> (integer), attributes (C<attr>, C<id>, C<class>,
+C<classes>, C<keyvals>), and text (C<content>, a list of L<inlines|/INLINE ELEMENTS>).
 
     Header $level, $attributes, [ @inlines ]
 
@@ -559,7 +825,7 @@ list of L<inlines|/INLINE ELEMENTS>).
 
 Horizontal rule
 
-    HorizontalRule 
+    HorizontalRule
 
 =head3 Null
 
@@ -633,22 +899,26 @@ Citation, a list of C<citations> and a list of L<inlines|/INLINE ELEMENTS>
 
 =head3 Code
 
-Inline code, a literal string (C<content>) with attributes (C<attr>)
+Inline code, a literal string (C<content>) with attributes (C<attr>, C<id>,
+C<class>, C<classes>, C<keyvals>)
 
     Code attributes { %attr }, $content
 
 =head3 Emph
 
 Emphasized text, a list of L<inlines|/INLINE ELEMENTS> (C<content>).
- 
+
     Emph [ @inlines ]
 
 =head3 Image
 
 Image with alt text (C<content>, a list of L<inlines|/INLINE ELEMENTS>) and
-C<target> (list of C<url> and C<title>).
+C<target> (list of C<url> and C<title>) with attributes (C<attr>, C<id>,
+C<class>, C<classes>, C<keyvals>).
 
-    Image [ @inlines ], [ $url, $title ]
+    Image attributes { %attr }, [ @inlines ], [ $url, $title ]
+
+Serializing the attributes in JSON can be disabled with C<PANDOC_VERSION>.
 
 =head3 LineBreak
 
@@ -659,9 +929,12 @@ Hard line break
 =head3 Link
 
 Hyperlink with link text (C<content>, a list of L<inlines|/INLINE ELEMENTS>)
-and C<target> (list of C<url> and C<title>).
+and C<target> (list of C<url> and C<title>) with attributes (C<attr>, C<id>,
+C<class>, C<classes>, C<keyvals>).
 
-    Link [ @inlines ], [ $url, $title ]
+    Link attributes { %attr }, [ @inlines ], [ $url, $title ]
+
+Serializing the attributes in JSON can be disabled with C<PANDOC_VERSION>.
 
 =head3 Math
 
@@ -695,6 +968,18 @@ Small caps text, a list of L<inlines|/INLINE ELEMENTS> (C<content>).
 
     SmallCaps [ @inlines ]
 
+=head3 SoftBreak
+
+Soft line break
+
+    SoftBreak
+
+Note that the C<SoftBreak> element was added in Pandoc 1.16 to as a matter of
+editing convenience to preserve line breaks (as opposed to paragraph breaks)
+from input source to output. If you are going to feed a document containing
+C<SoftBreak> elements to Pandoc E<lt> 1.16 you will have to set the package
+variable or environment variable C<PANDOC_VERSION> to 1.15 or below.
+
 =head3 Space
 
 Inter-word space
@@ -704,7 +989,7 @@ Inter-word space
 =head3 Span
 
 Generic container of L<inlines|/INLINE ELEMENTS> (C<content>) with attributes
-(C<attr>).
+(C<attr>, C<id>, C<class>, C<classes>, C<keyvals>).
 
     Span attributes { %attr }, [ @inlines ]
 
@@ -712,7 +997,7 @@ Generic container of L<inlines|/INLINE ELEMENTS> (C<content>) with attributes
 
 Plain text, a string (C<content>).
 
-    Str $text
+    Str $content
 
 =head3 Strikeout
 
@@ -740,17 +1025,59 @@ Superscripted text, a list of L<inlines|/INLINE ELEMENTS> (C<content>).
 
 =head2 METADATA ELEMENTS
 
-=head3 MetaBlocks
+Metadata can be provided in YAML syntax or via command line option C<-M>.  All
+metadata elements return true for C<is_meta>.  Metadata elements can be
+converted to unblessed Perl array references, hash references, and scalars with
+method C<metavalue>.  On the document level, metadata (document method C<meta>)
+is a hash reference with values being metadata elements. Document method
+C<metavalue> returns a flattened copy of this hash.
+
+=head3 MetaString
+
+A plain text string metadata value (C<content>).
+
+    MetaString $content
+
+MetaString values can also be set via pandoc command line client:
+
+    pandoc -M key=$content
 
 =head3 MetaBool
 
+A Boolean metadata value (C<content>). The special values C<"false"> and
+C<"FALSE"> are recognized as false in addition to normal false values (C<0>,
+C<undef>, C<"">...).
+
+    MetaBool $content
+
+MetaBool values can also be set via pandoc command line client:
+
+    pandoc -M key=true
+    pandoc -M key=false
+
 =head3 MetaInlines
 
+Container for a list of L<inlines|/INLINE ELEMENTS> (C<content>) in metadata.
+
+    MetaInlines [ @inlines ]
+
+=head3 MetaBlocks
+
+Container for a list of L<blocks|/BLOCK ELEMENTS> (C<content>) in metadata.
+
+    MetaInlines [ @blocks ]
+
 =head3 MetaList
 
+A list of other L<metadata elements|/METADATA ELEMENTS> (C<content>).
+
+    MetaList [ @values ]
+
 =head3 MetaMap
 
-=head3 MetaString
+A map of keys to other metadata elements.
+
+    MetaMap { %map }
 
 =head2 DOCUMENT ELEMENT
 
@@ -761,6 +1088,12 @@ Root element, consisting of metadata hash (C<meta>) and document element array
 
     Document $meta, [ @blocks ]
 
+Document C<metavalue> returns a copy of the metadata hash with all L<metadata
+elements|/METADATA ELEMENTS> flattened to unblessed values:
+
+    $doc->metavalue   # equivalent to
+    { map { $_ => $doc->meta->{$_}->metavalue } keys %{$doc->meta} }
+
 =head2 TYPE KEYWORDS
 
 The following document elements are only as used as type keywords in other
@@ -778,11 +1111,11 @@ C<DisplayMath>, C<InlineMath>
 
 =item
 
-C<AuthorInText>, C<SuppressAuthor>, C<NormalCitation> 
+C<AuthorInText>, C<SuppressAuthor>, C<NormalCitation>
 
 =item
 
-C<AlignLeft>, C<AlignRight>, C<AlignCenter>, C<AlignDefault> 
+C<AlignLeft>, C<AlignRight>, C<AlignCenter>, C<AlignDefault>
 
 =item
 
@@ -801,12 +1134,16 @@ L<Pandoc> implements a wrapper around the pandoc executable.
 
 L<Text.Pandoc.Definition|https://hackage.haskell.org/package/pandoc-types/docs/Text-Pandoc-Definition.html>
 contains the original definition of Pandoc document data structure in Haskell.
-This module version was last aligned with pandoc-types-1.12.4.
+This module version was last aligned with pandoc-types-1.16.1.
 
 =head1 AUTHOR
 
 Jakob Voß E<lt>jakob.voss@gbv.deE<gt>
 
+=head1 CONTRIBUTORS
+
+Benct Philip Jonsson E<lt>bpjonsson@gmail.comE<gt>
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2014- Jakob Voß
